"""
Base classes used in transcoder.
"""

# SnakeBite - Transcodes GStreamer-supported audio to MP3
# Copyright (C) 2008 Jason Gerard DeRose <jderose@jasonderose.org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

from __future__ import absolute_import

import gobject
from pyawesome.core import Core
from pyawesome import util

log = Core.getLogger()



	
	
class AsyncBatchProcess(gobject.GObject):
	__gsignals__ = {
		'start': (
			gobject.SIGNAL_RUN_LAST, 
			gobject.TYPE_NONE, 
			[],
		),
		'finish': (
			gobject.SIGNAL_RUN_LAST, 
			gobject.TYPE_NONE, 
			[],
		),
		'progress': (
			gobject.SIGNAL_RUN_LAST,
			gobject.TYPE_NONE, 
			[
				gobject.TYPE_FLOAT, # Percent complete
				gobject.TYPE_STRING, # Textual message
			]
		),
		'changed': (
			gobject.SIGNAL_RUN_LAST,
			gobject.TYPE_NONE, 
			[
				gobject.TYPE_PYOBJECT,
			]
		),
	}
	progress_key = None
	allowed_statuses = None


	def __init__(self):
		super(AsyncBatchProcess, self).__init__()
		assert isinstance(self.progress_key, basestring)
		assert isinstance(self.allowed_statuses, list)
		for name in self.allowed_statuses:
			assert isinstance(name, basestring)
			
		self.__running = False
		self.__processing_item = False
		self.__total = 0
		self.__items = []
		self.__current = None
		
		self.__timer = util.Timer()
		self.__counter = util.Counter(self.allowed_statuses)
		
	
	
	def __get_running(self):
		"""
		Returns True if the AsyncBatchProcess is currently running,
		False otherwise.
		"""
		return self.__running
	running = property(__get_running)
	
	def __get_processing_item(self):
		"""
		Stuff.
		"""
		return self.__processing_item
	processing_item = property(__get_processing_item)
	
	def __get_n(self):
		"""
		Returns the number of the current item being processed (its index
		plus one).
		"""
		return self.total - self.remaining
	n = property(__get_n)
	
	def __get_remaining(self):
		"""
		Returns the number of items yet unprocessed.
		"""
		return len(self.__items)
	remaining = property(__get_remaining)
	
	def __get_percent(self):
		"""
		Returns the percent complete (floating point number between
		0.0 and 1.0).
		"""
		return self.n / float(self.total)
	percent = property(__get_percent)
	
	def __get_total(self):
		"""
		Returns the len() of the original items list passed.
		"""
		return self.__total
	total = property(__get_total)
	
	def __get_current(self):
		"""
		Returns the current item in process.
		"""
		return self.__current
	current = property(__get_current)
	
	def __get_counter(self):
		return self.__counter.proxy
	counter = property(__get_counter)
	
	def __get_timer(self):
		return self.__timer.proxy
	timer = property(__get_timer)
	
	def __next(self):
		gobject.idle_add(self.__next_idle)	

	def __next_idle(self):
		"""
		If still running and items remain, process the next item.
		Otherwise terminate processing.
		"""
		assert self.__counter.get_sum(self.remaining) == self.total
		if self.__running and len(self.__items) > 0:
			self.__current = self.__items.pop()
			self.__emit_progress()
			self.emit('changed', self.__current)
			self.__processing_item = True
			self.process()
		else:
			self.__running = False
			self.emit('finish')
		return False # Don't repeate idle call
		


	def __emit_progress(self):
		"""
		Emit a progress signal.
		"""
		msg = Core.text(
			self.progress_key,
			current=self.n,
			total=self.total,
		)
		log.info(msg)
		self.emit('progress', self.percent, msg)
		
		
	### PUBLIC API METHODS ###
	
	def abort(self):
		"""
		Sets __running to False and then calls abort_process() in the
		subclass.
		
		The subclass must still call process_finished().
		"""
		self.__running = False
		if self.__processing_item:
			self.abort_process()
	

	def run(self, items):
		assert not self.__running
		self.__running = True
		self.__items = list(items)
		self.__items.reverse()
		self.__total = len(self.__items)
		self.__current = None
		self.__counter.reset()
		self.__timer.reset()
		self.__timer.start()
		self.emit('start')
		self.emit('progress', 0.0, '')
		self.__next()


	def finished_process(self, status):
		"""
		Should be called when process() is finished.  The single
		argument should be the exit status of process().
		
		Causes the next item to be processed.
		"""
		log.info('finished_process(%r)' % status)
		self.__processing_item = False
		self.__counter.increment(status)
		self.__next()
		

		

	### MUST BE IMPLEMENTED IN SUBCLASSES ###	
	def process(self):
		"""
		Process the self.current.  Override in subclasses.
		"""
		raise NotImplementedError('process_item')
		
	def abort_process(self):
		"""
		Abort the processing of self.current.  Override in subclasses.
		"""
		raise NotImplementedError('abort_process_item')

