"""
GTK2 GUI for SnakeBite.
"""

# SnakeBite - Transcodes GStreamer-supported audio to MP3
# Copyright (C) 2008 Jason Gerard DeRose <jderose@jasonderose.org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

from __future__ import absolute_import

import gobject, gtk, pango
import os
from os import path
from xml.sax import saxutils

from pyawesome.core import Core
from pyawesome.config import GconfValue

from .constants import (
	PADDING,
	ICON_SIZE,
	WINDOW_WIDTH,
	WINDOW_HEIGHT,
	MP3_BITRATES,
)
from .transcoder import BatchTranscoder
from .info import info



class TextKey(object):
	_prefix = None
	
	def set_text_key(self, key, **kw):
		assert isinstance(key, basestring)
		if self._prefix is not None:
			key = '%s_%s' % (self._prefix, key)
		self.set_text(Core.text(key, **kw))
		


class Label(gtk.Label, TextKey):
	_prefix = 'label'
	
	def __init__(self, bold=False):
		super(Label, self).__init__()
		self.__bold = bold
		
	def set_text(self, text):
		if self.__bold:
			self.set_markup('<b>%s</b>' % saxutils.escape(text))
		else:
			super(Label, self).set_text(text)
			
class SummaryLabel(Label):
	_prefix = 'summary'
	
class Frame(gtk.Frame):
	def __init__(self):
		gtk.Frame.__init__(self)
		self._framelabel = Label(bold=True)
		self._framelabel.set_padding(5, 5)
		self.set_label_widget(self._framelabel)
		self.set_label_align(0.5, 0.5)
		
	def set_sensitive(self, value):
		child = self.get_child()
		if child != None:
			child.set_sensitive(value)
			
	def set_text_key(self, key, **kw):
		self._framelabel.set_text_key(key, **kw)



class Button(gtk.Button, TextKey):
	_prefix = 'button'

	def __init__(self, stock=None, key=None):
		super(Button, self).__init__()
		hbox = gtk.HBox()
		self.add(hbox)
		self._image = gtk.Image()
		self._label = Label()
		self._label.set_padding(0, 0)
		self._label.set_alignment(0, 0.5)
		hbox.pack_start(self._image, False, False, PADDING)
		hbox.pack_start(self._label, True, True, PADDING)
		if stock != None:
			self.set_stock(stock)
		if key != None:
			self.set_text_key(key)
		
	def set_text(self, text):
		self._label.set_text(text)
		
	def set_stock(self, stock, size=ICON_SIZE):
		self._image.set_from_stock(stock, size)
		
		
class GconfLabel(gtk.Label, GconfValue):
	def __init__(self, gconf_key, processor=None):
		gtk.Label.__init__(self)
		GconfValue.__init__(self, Core.config, gconf_key)
		assert processor is None or callable(processor)
		self.__processor = processor
	
	def gconf_value_changed(self):
		"""
		Overrides GconfValue.gconf_value_changed()
		"""
		if self.__processor is None:
			text = str(self.gconf_value)
		else:
			text = self.__processor(self.gconf_value)
		self.set_text(text)


class QualitySlider(gtk.HScale, GconfValue):
	def __init__(self, gconf_key, vmin, vmax, digits):
		gtk.HScale.__init__(self)
		GconfValue.__init__(self, Core.config, gconf_key)
		self.set_range(vmin, vmax)
		self.set_digits(digits)
		self.set_value_pos(gtk.POS_RIGHT)
		self.set_draw_value(True)
		self.smart_connect(
			self,
			'value-changed',
			self.__on_value_changed
		)
		
	def gconf_value_changed(self):
		"""
		Overrides GconfValue.gconf_value_changed()
		"""
		self.set_value(self.gconf_value)
		
	def __on_value_changed(self, scale):
		self.gconf_value = scale.get_value()



class FolderChooser(Button, GconfValue):
	__gsignals__ = {
		'changed': (
			gobject.SIGNAL_RUN_LAST, 
			gobject.TYPE_NONE,
			[
				gobject.TYPE_STRING,
			]
		),
	}

	def __init__(self, key):
		assert isinstance(key, str)
		Button.__init__(self, stock=gtk.STOCK_OPEN)
		GconfValue.__init__(self, Core.config, key)
		self.__dialog_key = 'dialog_%s' % key
		self._label.set_ellipsize(pango.ELLIPSIZE_START)
		self.connect('clicked', self.__on_clicked)
		
	def gconf_value_changed(self):
		"""
		Overrides GconfValue.gconf_value_changed()
		"""
		self._label.set_text(self.gconf_value)
			
	def __on_clicked(self, button):
		dialog = gtk.FileChooserDialog(
			title=Core.text(self.__dialog_key),
			action=gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER, 
			buttons=(
				gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, 
				gtk.STOCK_OPEN, gtk.RESPONSE_OK,
			),
		)
		val = self.gconf_value
		folder = (val if path.isdir(val) else Core.dirs['home'])
		dialog.set_current_folder(folder)
		response = dialog.run()
		if response == gtk.RESPONSE_OK:
			self.gconf_value = dialog.get_filename()
			self.emit('changed', self.gconf_value)
		dialog.destroy()
		
		
class Table(gtk.Table):
	def __init__(self):
		super(Table, self).__init__()
		self._row = 0

	def _attach_helper(self, widget, col, **kw):
		x_options = gtk.FILL
		y_options = gtk.FILL
		if kw.get('x_expand', False):
			x_options |= gtk.EXPAND
		if kw.get('y_expand', False):
			y_options |= gtk.EXPAND
		colspan = kw.get('colspan', 1)
		self.attach(
			widget,
			col,
			col + colspan, 
			self._row,
			self._row + 1,
			x_options,
			y_options,
			PADDING,
			PADDING,
		)
	
	def _labeled_row(self, key, widget, bold=False):
		label = Label(bold=bold)
		label.set_alignment(1.0, 0.5)
		label.set_padding(2, 6)
		label.set_text_key(key)
		self._normal_row(label, widget)
		
	def _centered_label(self, key, bold=True):
		label = Label(bold=bold)
		label.set_padding(0, 10)
		label.set_text_key(key)
		self._attach_helper(label, 0, colspan=2)
		self._row += 1
		
	def _expanded_space(self):
		label = gtk.Label()
		self._attach_helper(label, 0,
			colspan=2,
			y_expand=True,
		)
		self._row += 1
		
	def _normal_row(self, left=None, right=None):
		if left is not None:
			self._attach_helper(left, 0)
		if right is not None:
			self._attach_helper(right, 1, x_expand=True)
		self._row += 1
		

		
	
class ProgressBar(gtk.ProgressBar):
	def __init__(self, obj, signal='progress'):
		super(ProgressBar, self).__init__()
		self._obj = obj
		self._obj.connect(signal, self.__on_progress)
		
	def __on_progress(self, obj, percent, msg):
		self.set_fraction(percent)
		self.set_text(msg)


class RadioSet(gtk.HBox, GconfValue):
	def __init__(self, gconf_key, values):
		gtk.HBox.__init__(self)
		GconfValue.__init__(self, Core.config, gconf_key)
		self.__radios = {}
		radio = None
		for value in values:
			radio = gtk.RadioButton(radio)
			self.__radios[value] = radio
			radio.set_label(Core.text('bitrate', kbps=value))
			radio.connect('toggled', self.__on_toggled, value)
			self.pack_start(radio)
			
	def __on_toggled(self, radio, value):
		if radio.get_active():
			self.gconf_value = value
			
	def gconf_value_changed(self):
		self.__radios[self.gconf_value].set_active(True)
			

class Options(Table):
	def __init__(self):
		super(Options, self).__init__()
		self.src = self.folder_chooser('src')
		self.dst = self.folder_chooser('dst')
		self._labeled_row(
			'mp3_bitrate',
			RadioSet('mp3_bitrate', MP3_BITRATES),
		)
		
	def folder_chooser(self, key):
		fc = FolderChooser(key)
		self._labeled_row(key, fc)
		return fc
		
class Progress(Table):
	def __init__(self, transcoder):
		super(Progress, self).__init__()
		self.__transcoder = transcoder
		transcoder.connect('changed', self.__on_change)

		self._centered_label('settings')
		self.__folder_label('src')
		self.__folder_label('dst')
		def process_bitrate(value):
			return Core.text('bitrate', kbps=value)	
		self.__folder_label('mp3_bitrate', processor=process_bitrate)
		
		#self._expanded_space()
		
		self._centered_label('progress')
		self.__current_folder = self.__current('current_folder')
		self.__current_file = self.__current('current_file', start=False)
		self.__progress_bar('file', 'progress_file')
		self.__progress_bar('total', 'progress')
		
	def __progress_bar(self, key, signal):
		pg = ProgressBar(self.__transcoder, signal)
		self._labeled_row(key, pg)
		
	def __current(self, key, start=True):
		label = gtk.Label()
		label.set_alignment(0, 0.5)
		if start:
			label.set_ellipsize(pango.ELLIPSIZE_START)
		else:
			label.set_ellipsize(pango.ELLIPSIZE_END)
		self._labeled_row(key, label)
		return label

	def __folder_label(self, key, processor=None):
		label  = GconfLabel(key, processor=processor)
		label.set_alignment(0, 0.5)
		label.set_ellipsize(pango.ELLIPSIZE_START)
		self._labeled_row(key, label)
		
	def __on_change(self, transcoder, current):
		cfolder = ('' if current is None else current.src_dir)
		cfile = ('' if current is None else current.src_name)
		self.__current_folder.set_text(cfolder)
		self.__current_file.set_text(cfile)
		

class StateBox(gtk.VBox):
	def __init__(self, transcoder, stock, action):
		super(StateBox, self).__init__()
		self.__transcoder = transcoder
		self.__action = action
		self.__button = Button(stock, action)
		self.__button.connect('clicked', self.__on_clicked)
		self.pack_end(self.__button, False, False, PADDING)
		
		
	def __on_clicked(self, button):
		assert button is self.__button
		getattr(self.__transcoder, self.__action)()
		
		




class IdleState(StateBox):
	def __init__(self, transcoder):
		super(IdleState, self).__init__(
			transcoder,
			gtk.STOCK_CONVERT,
			'transcode',
		)
		self.__options = Options()
		self.pack_start(self.__options, False, False, PADDING)
	
		label = gtk.Label()
		label.set_property('wrap', True)
		label.set_width_chars(65)
		label.set_markup(Core.text('markup_help'))
		frame = Frame()
		frame.set_text_key('help')
		frame.add(label)
		self.pack_start(frame, True, True, PADDING)
		
class ActiveState(StateBox):
	def __init__(self, transcoder):
		super(ActiveState, self).__init__(
			transcoder,
			gtk.STOCK_STOP,
			'abort',
		)
		self.__progress = Progress(transcoder)
		self.pack_start(self.__progress, True, True, PADDING)





class ClosingDialog(gtk.Dialog):
	def __init__(self, window, **kw):
		dkey = kw.get('dkey', 'dialog_close')
		mkey = kw.get('mkey', 'message_close')
		super(ClosingDialog, self).__init__(
			Core.text(dkey), # Title
			window, # Parent window
			buttons=(
				gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, 
				gtk.STOCK_CLOSE, gtk.RESPONSE_OK,
			),
		)
		hbox = gtk.HBox()
		image = gtk.Image()
		image.set_from_stock(
			gtk.STOCK_DIALOG_QUESTION,
			gtk.ICON_SIZE_DIALOG,
		)
		label = gtk.Label(Core.text(mkey))
		hbox.pack_start(image, False, False, PADDING)
		hbox.pack_start(label, False, False, PADDING)
		self.vbox.pack_start(hbox, False, False, PADDING)
		hbox.show_all()
		
def run_closing_dialog(window, **kw):
	dialog = ClosingDialog(window, **kw)
	response = dialog.run()
	dialog.destroy()
	if response == gtk.RESPONSE_OK:
		return False
	else:
		return True
		
		
class SummaryDialog(gtk.Dialog):
	def __init__(self, window, transcoder, **kw):
		dkey = kw.get('dkey', 'dialog_summary')
		mkey = kw.get('dkey', 'message_summary')
		super(SummaryDialog, self).__init__(
			Core.text(dkey), # Title
			window, # Parent window
			buttons=(
				gtk.STOCK_CLOSE, gtk.RESPONSE_OK,
			),
		)
		hbox = gtk.HBox()
		image = gtk.Image()
		image.set_from_stock(
			gtk.STOCK_DIALOG_INFO,
			gtk.ICON_SIZE_DIALOG,
		)
		
		
		label = gtk.Label(Core.text(mkey, time_elapsed=transcoder.timer.minsec))
		hbox.pack_start(image, False, False, PADDING)
		hbox.pack_start(label, False, False, PADDING)
		self.vbox.pack_start(hbox, False, False, PADDING)
		
		label = self.__add_label(bold=True)
		label.set_text_key('total', count=transcoder.total)
		
		counter = transcoder.counter
		for key in transcoder.allowed_statuses:
			if counter.get_count(key) < 1:
				continue
			label = self.__add_label()
			label.set_text_key(key, count=counter.get_count(key))
			
		if transcoder.remaining > 0:
			label = self.__add_label()
			label.set_text_key('remaining', count=transcoder.remaining)

		self.vbox.show_all()
		
	def __add_label(self, bold=False):
		label = SummaryLabel(bold=bold)
		label.set_alignment(0, 0.5)
		self.vbox.pack_start(label, False, False, PADDING)
		return label


class GUI(object):
	def __init__(self):
		self.transcoder = BatchTranscoder()
		self.transcoder.connect('start', self.on_start)
		self.transcoder.connect('finish', self.on_finish)
	
		self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
		self.window.connect('delete_event', self.on_delete_event)
		self.window.connect('destroy', self.on_destroy)
		self.window.set_default_size(WINDOW_WIDTH, WINDOW_HEIGHT)
		
		self.set_title_active()
		
		box = gtk.HBox()
		self.window.add(box)
		
		self.idle = IdleState(self.transcoder)
		box.pack_start(self.idle, True, True, PADDING)
		
		self.active = ActiveState(self.transcoder)
		box.pack_start(self.active, True, True, PADDING)
		
		
		self.window.show_all()
		self.window.resize(WINDOW_WIDTH, WINDOW_HEIGHT)
		self.active.hide()
		
	def set_title_active(self):
		self.window.set_title('%s %s' % (
				Core.text('title_idle'),
				info['version'],
			)
		)
		
	def on_start(self, transcoder):
		assert transcoder is self.transcoder
		self.idle.hide()
		self.active.show()
		self.window.set_title(Core.text('title_active'))
		
	def on_finish(self, transcoder):
		assert transcoder is self.transcoder
		self.active.hide()
		self.idle.show()
		self.set_title_active()
		d = SummaryDialog(self.window, self.transcoder)
		d.run()
		d.destroy()
		
		
	def on_convert_clicked(self, button):
		assert not self.transcoder.running
		self.transcoder.transcode(
			self.options.src.gconf_value,
			self.options.dst.gconf_value,
		)

	
	def on_delete_event(self, window, event):
		if not self.transcoder.running:
			return False
		return run_closing_dialog(window)


		
	def on_destroy(self, window):
		self.transcoder.abort()
		gtk.main_quit()
		
	def start(self):
		gtk.main()
		
		
		
		
		
		
		
		
		
		
		
		
		
		
		
		
		
		
		
		
		
		
		
		
		
		
		
		
		
		
		
		
		
		
		
