"""
SnakeBite - Transcodes GStreamer-supported audio to MP3
"""

# SnakeBite - Transcodes GStreamer-supported audio to MP3
# Copyright (C) 2008 Jason Gerard DeRose <jderose@jasonderose.org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

from __future__ import absolute_import

from pyawesome.core import Core
import os
from os import path
from shutil import copyfile
import gobject, gst
from gst.extend.discoverer import Discoverer
import shutil

from .base import AsyncBatchProcess
from .util import Periodic, Timer
log = Core.getLogger()

def format_time(sec):
	return '%d:%02d' % (sec / 60, sec % 60)
	
def format_posdur(pos, dur):
	return '%s / %s' % (format_time(pos), format_time(dur))
	
	
GST_FLOAT_SECOND = float(gst.SECOND)


class FileInfo(object):
	dst_ext = '.mp3'
	def __init__(self, src, src_base, dst_base):
		assert isinstance(src, basestring)
		assert isinstance(src_base, basestring)
		assert isinstance(dst_base, basestring)
		
		self.src = path.abspath(src)
		self.src_base = path.abspath(src_base)
		self.dst_base = path.abspath(dst_base)
		assert self.src.startswith(self.src_base)
		assert self.src_base != self.dst_base
		
		(self.src_dir, self.src_name) = path.split(self.src)
		
	
		# src_dir relative to src_base
		self.rel_dir = self.src_dir[len(self.src_base)+1:]
		assert not self.rel_dir.startswith('/')
		assert path.join(
			self.src_base, 
			self.rel_dir, 
			self.src_name,
		) == self.src
		
		
		(self.base_name, self.src_ext) = path.splitext(self.src_name)
		self.dst_name = self.base_name + self.dst_ext
		
		self.dst_dir = path.join(self.dst_base, self.rel_dir)
		self.dst = path.join(self.dst_dir, self.dst_name)

	def __get_tmp(self):
		return path.join(Core.dirs.tmp_dir, self.dst_name)
	tmp = property(__get_tmp)


def scan_files(src_dir, dst_dir):
	src_dir = path.abspath(src_dir)
	dst_dir = path.abspath(dst_dir)
	for (parent, dirs, files) in os.walk(src_dir):
		files.sort()
		for name in files:
			fullname = os.path.join(parent, name)
			if fullname.startswith(dst_dir):
				continue
			yield FileInfo(fullname, src_dir, dst_dir)




	

		
class BatchTranscoder(AsyncBatchProcess):
	__gsignals__ = {
		'progress_file': (
			gobject.SIGNAL_RUN_LAST,
			gobject.TYPE_NONE, 
			[
				gobject.TYPE_FLOAT, # Percent complete
				gobject.TYPE_STRING, # Textual message
			]
		),
	}
	allowed_statuses = [
		'success',
		'mp3_hardlinked',
		'mp3_copied',
		'preexisting',
		'non_audio',
		'transcode_error',
		'transcode_aborted',

	]
	__discoverer = None
	__transcoder = None
	progress_key = 'progress_total'

	

	
	
	def transcode(self):
		src_dir = Core.config['src']
		dst_dir = Core.config['dst']
		self.__bitrate = Core.config['mp3_bitrate']
		self.run(scan_files(src_dir, dst_dir))
		
	def abort_process(self):
		if self.__transcoder is None:
			self.finished_process('transcode_aborted')
		else:
			self.__transcoder.abort()
			
		
	def process(self):
		assert isinstance(self.current, FileInfo)
		self.emit('progress_file', 0.0, '')
		if path.exists(self.current.dst):
			log.info('Not over-writing %r' % self.current.dst)
			self.finished_process('preexisting')
		else:
			self.__discover_current()
			
	def __discover_current(self):
		assert self.__discoverer is None
		assert isinstance(self.current, FileInfo)
		log.info('Discovering %s' % self.current.src)
		self.__discoverer = Discoverer(self.current.src)
		self.__discoverer.connect('discovered', self.__on_discovered)
		self.__discoverer.discover()
		
	def __on_discovered(self, d, is_media):
		assert d is self.__discoverer
		is_audio = (is_media and d.is_audio and not d.is_video)
		self.__discoverer = None
		if is_audio:
			# FIXME: This is not the way to do this, but Discoverer
			# doesn't seem expose what we need to effectively
			# determine whether something is really an mp3.
			# So this hack will work for testing for now.
			if self.current.src_ext.lower() == '.mp3':
				self.__hardlink_current()
			else:
				self.__transcode_current()
		else:
			self.finished_process('non_audio')
			

	def __hardlink_current(self):
		assert isinstance(self.current, FileInfo)
		if path.exists(self.current.dst):
			self.finished_process('preexisting')
			return
		if not path.isdir(self.current.dst_dir):
			log.info('Creating directory %r' % self.current.dst_dir)
			os.makedirs(self.current.dst_dir)
		try:
			os.link(self.current.src, self.current.dst)
			log.info('Created hardlink %r pointing to %r' % (
					self.current.dst, 
					self.current.src
				)
			)
			self.finished_process('mp3_hardlinked')
		except OSError:
			self.finished_process('mp3_copied')
			log.info('Copying file %r to %r' % (
					self.current.src,
					self.current.dst, 
				)
			)
			shutil.copyfile(self.current.src, self.current.dst)
		
		
	def __transcode_current(self):
		assert self.__transcoder is None
		assert isinstance(self.current, FileInfo)
		log.info('Transcoding %s' % self.current.src)
		self.__transcoder = Transcoder(self.current, self.__bitrate)
		self.__transcoder.connect('transcoded', self.__on_transcoded)
		self.__transcoder.connect('posdur', self.__on_posdur)
		self.__transcoder.transcode()
		
	def __on_transcoded(self, t, status):
		assert t is self.__transcoder
		self.__transcoder = None
		self.finished_process(status)
		
	def __on_posdur(self, t, pos, dur):
		assert t is self.__transcoder
		percent = (pos / float(dur) if dur > 0 else 0.0)
		self.emit(
			'progress_file',
			percent,
			Core.text(
				'progress_file',
				time_completed=format_time(pos),
				time_total=format_time(dur),
			),
		)

class Transcoder(gobject.GObject):
	__gsignals__ = {
		'transcoded': (
			gobject.SIGNAL_RUN_LAST, 
			gobject.TYPE_NONE, 
			[
				gobject.TYPE_STRING,
			],
		),
		'posdur': (
			gobject.SIGNAL_RUN_LAST,
			gobject.TYPE_NONE, 
			[
				gobject.TYPE_FLOAT, # Percent complete
				gobject.TYPE_FLOAT, # Textual message
			]
		),
	}
	
	__running = False
	__allowed_statuses = set([
		'success',
		'preexisting',
		'transcode_error',
		'transcode_aborted',
	])
	
	def __init__(self, f, bitrate):
		super(Transcoder, self).__init__()
		assert isinstance(f, FileInfo)
		assert isinstance(bitrate, int)
		log.info('Transcoding %r to temp file %r' % (f.src, f.tmp))
		self.f = f
		self.periodic = Periodic(1000, self.on_periodic)
		
		# The pipeline
		self.pipeline = gst.Pipeline()
		
		# Create bus and connect several handlers
		self.bus = self.pipeline.get_bus()
		self.bus.add_signal_watch()
		self.bus.connect('message::eos', self.on_eos)
		self.bus.connect('message::error', self.on_error)

		# Create elements
		self.src = gst.element_factory_make('filesrc')
		self.dec = gst.element_factory_make('decodebin2')
		self.conv = gst.element_factory_make('audioconvert')
		self.rsmpl = gst.element_factory_make('audioresample')
		self.q = gst.element_factory_make('queue')
		self.enc = gst.element_factory_make('lame')
		self.sink = gst.element_factory_make('filesink')
		
		# Set properties
		self.src.set_property('location', f.src)
		self.enc.set_property('bitrate', bitrate)
		log.info('mp3 bitrate = %d' % self.enc.get_property('bitrate'))
		self.sink.set_property('location', f.tmp)
		
		# Connect handler for 'new-decoded-pad' signal 
		self.dec.connect('new-decoded-pad', self.on_new_decoded_pad)
		
		# Add elements to pipeline
		self.pipeline.add(
			self.src,
			self.dec,
			self.conv,
			self.rsmpl,
			self.q,
			self.enc, 
			self.sink,
		)
		
		# Link *some* elements 
		# This is completed in self.on_new_decoded_pad()
		self.src.link(self.dec)
		gst.element_link_many(
			self.conv,
			self.rsmpl,
			self.q,
			self.enc, 
			self.sink,
		)
		
		# Reference used in self.on_new_decoded_pad()
		self.apad = self.conv.get_pad('sink')


	
	def transcode(self):
		self.__running = True
		self.timer = Timer()
		self.periodic.start()
		self.pipeline.set_state(gst.STATE_PLAYING)
		
	def on_periodic(self):
		if not self.__running:
			return
		(pos, dur) = self.get_posdur()
		self.emit('posdur', pos, dur)	


	def get_posdur(self):
		"""
		Returns a (position, duration) tuple representing the position
		and duration of the current stream in seconds, as floating
		point numbers.
		"""
		try:
			(pos, format) = \
				self.pipeline.query_position(gst.FORMAT_TIME)
			(dur, format) = \
				self.pipeline.query_duration(gst.FORMAT_TIME)
			return (
				pos/GST_FLOAT_SECOND,
				dur/GST_FLOAT_SECOND,
			)
		except gst.QueryError:
			return (0.0, 0.0)
		
	def on_new_decoded_pad(self, element, pad, last):
		caps = pad.get_caps()
		name = caps[0].get_name()
		if name.startswith('audio/') and not self.apad.is_linked():
			pad.link(self.apad)

			
			
	def on_eos(self, bus, msg):
		log.info('Transcoded %r in %s' % (self.f.src, self.timer.minsec))
		if not path.exists(self.f.dst):
			if not path.isdir(self.f.dst_dir):
				log.info('Created directory %r' % self.f.dst_dir)
				os.makedirs(self.f.dst_dir)
			log.info('Moving %r to %r' % (self.f.tmp, self.f.dst))
			shutil.move(self.f.tmp, self.f.dst)
			self.finish('success')
		else:
			log.warn(
				'Destination file %r already exists; deleting %r' % (
					 self.f.dst, 
					 self.f.tmp,
				)
			)
			os.unlink(self.f.tmp)
			self.finish('preexisting')
			
	def abort(self):
		self.finish('transcode_aborted')
							
	def on_error(self, bus, msg):
		error = msg.parse_error()
		log.error('Error transcoding %r: %r' % (self.f.src, error[1]))
		self.finish('transcode_error')
		
	def finish(self, status):
		assert status in self.__allowed_statuses
		self.__running = False
		self.periodic.stop()
		self.pipeline.set_state(gst.STATE_NULL)
		self.emit('transcoded', status)

