"""
Various utility functions and classes.
"""

# SnakeBite - Transcodes GStreamer-supported audio to MP3
# Copyright (C) 2008 Jason Gerard DeRose <jderose@jasonderose.org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

from __future__ import absolute_import
from pyawesome.core import Core
import gobject
import time

log = Core.getLogger()


class Periodic(object):
	"""
	Repeatedly calls callback() with a period of ms milliseconds.
	
	Uses gobject.timeout_add, so the gobject.MainLoop() must be running.
	"""
	
	def __init__(self, ms, callback):
		assert isinstance(ms, int)
		assert callable(callback)
		self.__ms = ms
		self.__callback = callback
		self.__timeout_id = None

	def __on_timeout(self):
		self.__callback()
		return True # Repeat timeout call
	
	def start(self):
		"""
		Start periodic timer if it isn't already started.
		
		Returns False if timer was already started, otherwise True.
		"""
		if self.__timeout_id is None:
			self.__timeout_id = gobject.timeout_add(
				self.__ms,
				self.__on_timeout,
			)
			return True
		return False
		
	def stop(self):
		"""
		Stop periodic timer if it isn't already stopped.
		
		Returns False if timer was already stopped, otherwise True.
		"""
		if self.__timeout_id is not None:
			gobject.source_remove(self.__timeout_id)
			self.__timeout_id = None
			return True
		return False
			
	def restart(self):
		"""
		Restart periodic timer if it is currently running.
		"""
		if self.stop():
			self.start()


	
class Timer(object):
	"""
	Keeps track of elapsed time (in seconds).
	"""
	
	def __init__(self):
		self.__start = time.time()
		
	def __get_elapsed(self):
		"""
		Returns elapsed time in seconds as a float.
		"""
		return time.time() - self.__start
	elapsed = property(__get_elapsed)
	
	def __get_minsec(self):
		"""
		Returns (minutes, seconds) tuple representing current elapsed
		time.
		"""
		sec = int(self.elapsed)
		return '%d:%02d' % (sec / 60, sec % 60)
	minsec = property(__get_minsec)


